Imports System.IO
Imports System.Text
Imports System.Security.Cryptography

Module CryptoStuff
    ' Return bytes representing an encrypted string.
    Public Function EncryptString(ByVal password As String, ByVal salt() As Byte, ByVal iterations As Integer, ByVal plaintext As String) As Byte()
        ' Make a key provider.
        Dim key_provider As New Rfc2898DeriveBytes(password, salt, iterations)

        ' Make a triple DES provider.
        Dim triple_des As TripleDES = TripleDES.Create()

        ' Set the key.
        Dim key_size_bytes As Integer = triple_des.KeySize() \ 8
        triple_des.Key = key_provider.GetBytes(key_size_bytes)

        ' Set the initialization vector.
        Dim iv_size_bytes As Integer = triple_des.BlockSize() \ 8
        triple_des.IV = key_provider.GetBytes(iv_size_bytes)

        ' Encrypt the string.
        Dim encrypted_stream As New MemoryStream()
        Dim crypto_stream As New CryptoStream(encrypted_stream, _
            triple_des.CreateEncryptor(), CryptoStreamMode.Write)
        Dim plaintext_bytes() As Byte = New System.Text.UTF8Encoding(False).GetBytes(plaintext)

        crypto_stream.Write(plaintext_bytes, 0, plaintext_bytes.Length)
        crypto_stream.FlushFinalBlock()
        crypto_stream.Close()

        Return encrypted_stream.ToArray()
    End Function

    ' Return bytes representing an encrypted string.
    Public Function DecryptString(ByVal password As String, ByVal salt() As Byte, ByVal iterations As Integer, ByVal ciphertext_bytes() As Byte) As String
        ' Make a key provider.
        Dim key_provider As New Rfc2898DeriveBytes(password, salt, iterations)

        ' Make a triple DES provider.
        Dim triple_des As TripleDES = TripleDES.Create()

        ' Set the key.
        Dim key_size_bytes As Integer = triple_des.KeySize() \ 8
        triple_des.Key = key_provider.GetBytes(key_size_bytes)

        ' Set the initialization vector.
        Dim iv_size_bytes As Integer = triple_des.BlockSize() \ 8
        triple_des.IV = key_provider.GetBytes(iv_size_bytes)

        ' Decrypt the string.
        Dim decrypted_stream As New MemoryStream()
        Dim crypto_stream As New CryptoStream(decrypted_stream, _
            triple_des.CreateDecryptor(), CryptoStreamMode.Write)

        crypto_stream.Write(ciphertext_bytes, 0, ciphertext_bytes.Length)
        crypto_stream.FlushFinalBlock()
        crypto_stream.Close()

        Dim result As String = New UTF8Encoding(False).GetString( _
            decrypted_stream.ToArray())
        Return result
    End Function
End Module
